﻿Imports System.Drawing.Drawing2D

Public Class Form1
    ' The polylines we are drawing.
    Private Polylines As New List(Of List(Of Point))()
    Private NewPolyline As List(Of Point) = Nothing

    ' Exit.
    Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
        Me.Close()
    End Sub

    ' Clear old polylines.
    Private Sub mnuFileNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileNew.Click
        Polylines = New List(Of List(Of Point))()
        picCanvas.Refresh()
    End Sub

    ' Start drawing.
    Private Sub picCanvas_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseDown
        NewPolyline = New List(Of Point)()
        Polylines.Add(NewPolyline)
        NewPolyline.Add(e.Location)
    End Sub

    ' Continue drawing.
    Private Sub picCanvas_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseMove
        If (NewPolyline Is Nothing) Then Return
        NewPolyline.Add(e.Location)
        picCanvas.Refresh()
    End Sub

    ' Stop drawing.
    Private Sub picCanvas_MouseUp(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseUp
        If (NewPolyline Is Nothing) Then Return
        NewPolyline = Nothing
        picCanvas.Refresh()
    End Sub

    ' Draw the polylines.
    Private Sub picCanvas_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles picCanvas.Paint
        e.Graphics.SmoothingMode = SmoothingMode.AntiAlias

        ' Make a list of transformations, starting with the identity.
        Dim matrices As New List(Of Matrix)()
        matrices.Add(New Matrix())

        ' Make transformation matrices for the selected style.
        Dim wid As Integer = picCanvas.ClientSize.Width
        Dim hgt As Integer = picCanvas.ClientSize.Height
        Dim src_rect As New Rectangle(0, 0, wid, hgt)
        If (mnuStyleReflectX.Checked OrElse mnuStyleReflectXY.Checked) Then
            ' Reflect across X axis.
            Dim pts As Point() = {New Point(wid, 0), New Point(0, 0), New Point(wid, hgt)}
            Dim mat As New Matrix(src_rect, pts)
            matrices.Add(mat)
        End If
        If (mnuStyleReflectY.Checked OrElse mnuStyleReflectXY.Checked) Then
            ' Reflect across Y axis.
            Dim pts As Point() = {New Point(0, hgt), New Point(wid, hgt), New Point(0, 0)}
            Dim mat As New Matrix(src_rect, pts)
            matrices.Add(mat)
        End If
        If (mnuStyleReflectXY.Checked) Then
            ' Reflect across X and Y axes.
            Dim pts As Point() = {New Point(wid, hgt), New Point(0, hgt), New Point(wid, 0)}
            Dim mat As New Matrix(src_rect, pts)
            matrices.Add(mat)
        End If
        If (mnuStyleRotate2.Checked) Then
            ' Rotate 180 degrees.
            Dim mat As New Matrix()
            mat.RotateAt(180, New PointF(CSng(wid / 2), CSng(hgt / 2)))
            matrices.Add(mat)
        End If
        If (mnuStyleRotate4.Checked) Then
            ' Rotate 90 degrees three times.
            For i As Integer = 1 To 3
                Dim mat As New Matrix()
                mat.RotateAt(i * 90, New PointF(CSng(wid / 2), CSng(hgt / 2)))
                matrices.Add(mat)
            Next i
        End If
        If (mnuStyleRotate8.Checked) Then
            ' Rotate 45 degrees seven times.
            For i As Integer = 1 To 7
                Dim mat As New Matrix()
                mat.RotateAt(i * 45, New PointF(CSng(wid / 2), CSng(hgt / 2)))
                matrices.Add(mat)
            Next i
        End If

        ' Loop through all of the transformations.
        For Each mat As Matrix In matrices
            e.Graphics.Transform = mat
            For Each pline As List(Of Point) In Polylines
                e.Graphics.DrawLines(Pens.Blue, pline.ToArray())
            Next pline
        Next mat
    End Sub

    ' Uncheck all other menu items.
    Private Sub mnuStyle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuStyleReflectX.Click, mnuStyleRotate8.Click, mnuStyleRotate4.Click, mnuStyleRotate2.Click, mnuStyleReflectY.Click, mnuStyleReflectXY.Click
        Dim clicked As ToolStripMenuItem = DirectCast(sender, ToolStripMenuItem)

        clicked.Checked = True
        For Each item As ToolStripMenuItem In mnuStyle.DropDownItems
            If (item IsNot clicked) Then item.Checked = False
        Next item
    End Sub
End Class
